/*
m_sb_new - Load a bank of samples into consecutive ftables

DESCRIPTION
Read a number of audiofiles from one directory into a bunch of ftables.

SYNTAX
iStatus m_sb_new Sname, Sdirectory, Sextension [, iFirstNumber] [, iChannels]

INITIALIZATION
iStatus - exit status: 1 on success, 0 otherwise
Sname - Name of this samplebank
Sdirectory - Directory to search for audiofiles
Sextension - Extension of the audiofiles, i.e. wav or aiff
iFirstNumber - optional first table number to try. If omitted this defaults
	to 200
iChannels - Number of channels to read (default=0 all channels)

PERFORMANCE

CREDITS
Author: Jeanette C.
*/

opcode m_sb_new, i, SSSjo
	Sname, Sdirectory, Sextension, iFirstNumber, iChannels xin

	; Set up the basic variables
	SchannelName strcat "m_sb.", Sname

	; Initialise the output status
	iStatus init 1 ; success
	
	; Only perform if a samplebank of that name doesn't exist
	if chnget(strcat(SchannelName, ".Valid")) == 1 then
		prints "\nWARNING m_sb_new %s: this samplebank already exists,m doing nothing.", Sname
		iStatus = 0
	else
		; Set the first ftable number
		if iFirstNumber == -1 then
			iStartSearch = 200
		else
			iStartSearch = iFirstNumber
		endif

		; Search for an actually unused table
		iSearchIndex init iStartSearch
		while ftexists(iSearchIndex) == 1 do
			iSearchIndex += 1
		od

		; Find the files
		Sfiles[] directory Sdirectory, strcat(".", Sextension)
		iNumberOfFiles lenarray Sfiles
		if (iNumberOfFiles == 0) then
			iStatus = 0
		endif

		; Make sure there are enough free ftables
		iFreeIndex init 0
		while iFreeIndex < iNumberOfFiles do
			if (ftexists(iSearchIndex + iFreeIndex) == 1) then
				iFreeIndex = iNumberOfFiles
				iStatus = 0
			else
				iFreeIndex += 1
			endif
		od

		; Check that the pathname has a slash at the end
		iDirLen strlen Sdirectory
		if (strcmp(strsub(Sdirectory,(iDirLen -1)),"/") != 0) then
			Spath strcat Sdirectory, "/"
		else
			Spath = Sdirectory
		endif

		; Create a table to hold the samplerates of the samples
		iSamplerateFn ftgen 0, 0, -iNumberOfFiles, -2, 0

		; Function table holding channel count of each sample
		iChannelFn ftgen 0, 0, -iNumberOfFiles, -2, 0

		; Actually load the files
		if iStatus == 1 then
			iIndex init 0
			while (iIndex < iNumberOfFiles) do
				iDummyFt ftgen (iSearchIndex + iIndex), 0, 0, 1, strcat(Spath, Sfiles[iIndex]), 0, 0, iChannels
				tabw_i ftsr(iSearchIndex + iIndex), iIndex, iSamplerateFn
				tabw_i ftchnls(iSearchIndex + iIndex), iIndex, iChannelFn
				iIndex += 1
			od
			chnset 1, strcat(SchannelName, ".Valid")
			chnset iNumberOfFiles, strcat(SchannelName, ".Size")
			chnset iSearchIndex, strcat(SchannelName, ".Start")
			chnset 1, strcat(SchannelName, ".Groups")
			chnset 0, strcat(SchannelName, ".Current") ; current index into the bank
			chnset iSamplerateFn, strcat(SchannelName, ".Samplerates")
			chnset iChannelFn, strcat(SchannelName, ".Channels")
			chnset 0, strcat(SchannelName, ".CurGroup") ; current round-robin group
		endif
	endif

	xout iStatus
endop
