/*
m_sb_round - Play samples from a samplebank in round-robin fashion

DESCRIPTION
Play samples from a samplebank in round robin fashion. On each consecutive
call a random sample is chosen from one group, then the group is incremented.
If there is only one group, random samples will be played from the whole
samplebank.

SYNTAX
aOut1, aOut2 m_sb_round Sname, kPitch [, iMode]

INITIALIZATION
Sname - Name of the samplebank, as given to m_sb_new
iMode - Playback mode: 0=forwards 1=backkwards (default=0)

PERFORMANCE
kPitch - Playback factor of the sample, 1=normal, <1=slower, >1=faster

CREDITS
Author: Jeanette C.
*/

opcode m_sb_round, aa, Sko
	Sname, kPitch, iMode xin

	setksmps 1 ; we need the syncphasor again and compare to aSyncOut

	; Set up the basic channel name
	SchannelName strcat "m_sb.", Sname

	; Initialise the outputs
	aOut1 init 0
	aOut2 init 0

	; syncphasor parameters
	aPhasor init 0
	aSyncOut init 0
	aSyncIn init 0

	; Only perform if this samplebank is valid
	if chnget(strcat(SchannelName, ".Valid")) == 1 then
		; Retrieve basic information about this samplebank
		iSize chnget strcat(SchannelName, ".Size")
		iFirst chnget strcat(SchannelName, ".Start")
		iGroups chnget strcat(SchannelName, ".Groups")
		iGroupLength init ceil(iSize / iGroups) ; equal splitting of the bank

		; Calculate the sample to playt
		iCurGroup chnget strcat(SchannelName, ".CurGroup")

		; Set the group for the next call
		if (iCurGroup + 1) == iGroups then
			chnset 0, strcat(SchannelName, ".CurGroup")
		else
			chnset (iCurGroup + 1), strcat(SchannelName, ".CurGroup")
		endif
		iCurSample = round(linrand:i(iGroupLength)) + (iGroupLength * iCurGroup)

		; Make sure we're in limits of the bank
		if iCurSample >= iSize then
			iCurSample = (iSize -1)
		endif

		; Now get some sample information
		iChnls tab_i iCurSample, chnget(strcat(SchannelName, ".Channels")), 0
		iSamplerate tab_i iCurSample, chnget(strcat(SchannelName, ".Samplerates")), 0
		iLength = nsamp(iFirst + iCurSample) / iSamplerate
		kPlayFreq = kPitch / iLength
		kPlayCounter init 0 ; count number of play throughs
		aPhasor, aSyncOut syncphasor kPlayFreq, aSyncIn

		; Set the play counter
		if k(aSyncOut) == 1 then
			kPlayCounter += 1
		endif

		if kPlayCounter == 0 then
			if iChnls == 1 then ; Mono file
				aOut1 flooper2 1, kPitch, 0, iLength, 0, (iFirst + iCurSample), \
					0, iMode, 0, 0
				aOut2 = aOut1
			else ; Stereo sample
				aOut1, aOut2 flooper2 1, kPitch, 0, iLength, 0, \
					(iFirst + iCurSample), 0, iMode, 0, 0
			endif
		else ; We have played once, now silcne
			aOut1 = 0
			aOut2 = 0
		endif
	else ; not a valid samplebank
		prints "\nERROR m_sb_round %s: This samplebank is not valid. Was it deleted?", Sname
		aOut1 = 0
		aOut2 = 0
	endif

	; Output the audio
	xout aOut1, aOut2
endop
