<CsoundSynthesizer>
<CsOptions>
-o dac -d --messagelevel=2
</CsOptions>
; ==============================================
<CsInstruments>

sr	=	48000
ksmps	=	1
nchnls	=	2
0dbfs	=	1

#include "m_sb_new.udo"
#include "m_sb_delete.udo"
#include "m_sb_valid.udo"
#include "m_sb_get_start.udo"
#include "m_sb_get_size.udo"
#include "m_sb_get_groups.udo"
#include "m_sb_set_groups.udo"
#include "m_sb_oneshot.udo"
#include "m_sb_loop.udo"
#include "m_sb_waveseq.udo"
#include "m_sb_round.udo"

; Create a samplebank
instr Create
	Sname init "s1" ; the universal name
	Sfolder init "x" ; folder where to look for samples
	Sextension init "wav" ; extension (don't use .wav)
	iFirstFtable init 189 ; optional
	iChannels init 0 ; use all available channels of the files (mono/stereo)
	iStatus m_sb_new Sname, Sfolder, Sextension, iFirstFtable, iChannels

	if iStatus == 1 then
		prints "Samplebank %s was created successfully.", Sname
	else
		prints "Samplebank %s was not created successfully.", Sname
	endif

	; Check that this samplebank is valid: it was created
	ivalid m_sb_valid Sname

	; What is the first ftable number used
	istart m_sb_get_start Sname

	; What's the size of the samplebank (how many samples does it have)
	isize m_sb_get_size Sname

	; How many round-robin groups does it have (always one on a new bank)
	igroups m_sb_get_groups Sname
	prints "\nSamplebank %s is valid: %d, first ftable: %d, size %d, groups: %d", Sname, ivalid, istart, isize, igroups
endin

; Play a oneshot sample
; p4 = sample number
instr Oneshot
	iSampleNumber init p4
	Sname init "s1"
	kPitch init .7 ; pitch;playback speed, this is slower than normal
	iMode init 1

	; We leave iStart and iEnd to their defaults, sample will play from
	; start to finish

	; Play the sample
	aOut1, aOut2 m_sb_oneshot Sname, kPitch, iSampleNumber, iMode
	outs aOut1, aOut2
endin

; Play a sample with looping
;p4 = iMode ; loop mode 0=forward, 1=backwards, 2=alternating
instr Loop
	iMode init p4
	Sname init "s1"
	kPitch line .7, p3, 1.5 ; increase playback speed
	iPosition init -1 ; auto increment internally
	iStart init .2 ; change sample start

	; We will play samples to the end

	; Play the sound
	aOut1, aOut2 m_sb_loop Sname, kPitch, iMode, iPosition, iStart
	outs aOut1, aOut2
endin

; Use the samplebank as a wave sequencer, it will play through the whole
; bank
;p4 = kTriggerFreq
instr Waveseq
	kTriggerFreq init p4
	Sname init "s1"
	kPitch init .5 ; play each sample at half speed
	kTriggerDrift init .3 ; Randomise the trigger frequency between
		; .7 * kTriggerFreq and 1.3 * kTriggerFreq
	
	; Play the sound
	aOut1, aOut2 m_sb_waveseq Sname, kPitch, kTriggerFreq, kTriggerDrift
	outs aOut1, aOut2
endin

; Play round robin samples
instr Round
	Sname init "s1"
	kPitch init 1.5

	; We leave iMode to the default of 1

	; Play the sound
	aOut1, aOut2 m_sb_round Sname, kPitch
	outs aOut1, aOut2
endin

; Delete the samplebank
instr Delete
	Sname init "s1"
	m_sb_delete Sname
endin

; Check the validity of the samplebank
instr Check
	Sname init "s1"
	ivalid m_sb_valid Sname

	if ivalid == 1 then
		prints "Samplebank %s is valid", Sname
	else
		prints "Samplebank %s is invalid/deleted.", Sname
	endif
endin

</CsInstruments>
; ==============================================
<CsScore>
i"Create" 0 2 ; play it so long so you can read the messages
i"Oneshot" 2 2 0
i"Oneshot" 4 2 4
i"Loop" 6 3 0
i"Loop" 9 3 1
i"Loop" 12 3 2
i"Waveseq" 15 3 2
i"Waveseq" 18 3 4
; Play a few round robins
i"Round" 21 1
i. + .
i. . .
i. . .
i. . .
i. . .
i"Delete" 28 .1
i"Check" 29 2
e
</CsScore>zR
</CsoundSynthesizer>
